<?php
/**
 * @file media_kiwi6/includes/providers/emaudio/kiwi6.inc
 *   Provide support for the Kiwi6 provider to the emfield.module.
 */

/**
 * hook emaudio_PROVIDER_info
 * this returns information relevant to a specific 3rd party audio provider
 * @return array
 *   an array of strings requested by various admin and other forms
 *   'name' => the translated name of the provider
 *   'url' => the url to the main page for the provider
 *   'settings_description' => a description of the provider that will be posted in the admin settings form
 *   'supported_features' => an array of rows describing the state of certain supported features by the provider.
 *      These will be rendered in a table, with the columns being 'Feature', 'Supported', 'Notes'.
 */
function emaudio_kiwi6_info() {
  return array(
    'provider' => 'kiwi6',
    'name' => t('Kiwi6'),
    'url' => 'http://kiwi6.com/',
    'settings_description' => t('These settings specifically affect player displayed from <a href="@provider" target="_blank">Kiwi6</a>.', array('@provider' => 'http://kiwi6.com/')),
  );
}

/**
 * hook emaudio_PROVIDER_settings
 * this should return a subform to be added to the emvideo_settings() admin settings page.
 * note that a form field will already be provided, at $form['PROVIDER'] (such as $form['vimeo'])
 * so if you want specific provider settings within that field, you can add the elements to that form field.
 */

function emaudio_kiwi6_settings() {
  $form = array();

  $form['media_kiwi6_hostet_by_message'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display the "Hosted by kiwi6.com file hosting" message'),
    '#default_value' => variable_get('media_kiwi6_hostet_by_message', TRUE),
    );
  $form['media_kiwi6_download_link'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display the "Download mp3" link'),
    '#default_value' => variable_get('media_kiwi6_download_link', TRUE),
    );

  return $form;
}

/**
 * hook emaudio_PROVIDER_extract
 * this is called to extract the video code from a pasted URL or embed code.
 * @param $code
 *   an optional string with the pasted URL or embed code
 * @return
 *   either an array of regex expressions to be tested, or a string with the audio code to be used
 *   if the hook tests the code itself, it should return either the string of the audio code (if matched), or an empty array.
 *   otherwise, the calling function will handle testing the embed code against each regex string in the returned array.
 */
function emaudio_kiwi6_extract($code = '') {
  // http://kiwi6.com/file/eP4pu9euh7
  // http://k004.kiwi6.com/hotlink/eP4pu9euh7/some_file_name.mp3
  // <object type="application/x-shockwave-flash" data="http://kiwi6.com/swf/player.swf" id="audioplayer" height="24" width="290" allowscriptaccess="always"><param name="movie" value="http://kiwi6.com/swf/player.swf" /><param name="FlashVars" value="playerID=audioplayer&soundFile=http://k004.kiwi6.com/uploads/hotlink/eP4pu9euh7&titles=some_file_name.mp3" /><param name="quality" value="high" /><param name="menu" value="false" /><param name="allowscriptaccess" value="always" /><param name="wmode" value="transparent" /></object><div style="font:10px Arial,sans-serif;color:#aaa">Hosted by <a style="color:#999" href="http://kiwi6.com">kiwi6.com file hosting</a>.<a style="color:#999" href="http://kiwi6.com/file/eP4pu9euh7">Download mp3</a> - <a href="http://kiwi6.com">Free File Hosting</a>.</div>
  if (preg_match('@kiwi6.com/(file|uploads/hotlink|hotlink)/(\S\w+)@i',
                 $code, $matches)) {
    return $matches[2];
  }

  return '';
}

/**
 * Implement emaudio_PROVIDER_data().
 */
function emaudio_kiwi6_data($field, $item) {
  $data = array();

  $code = $item['value'];

  $url = media_kiwi6_file_url($item['value']);
  $response = drupal_http_request($url);

  if ($response->code == 301 || $response->code == 302) {
    $http_code = $response->redirect_code;
  }
  else {
    $http_code = $response->code;
  }

  if ($http_code == 200) {
    if ($s = preg_match('@soundFile=([\w\/\.\:]+)(?:&=\w)?&titles=(.[^\"]+)@i',
                        $response->data, $match)) {
      $data['soundFile'] = $match[1];
      $data['titles'] = $match[2];
    }
  }

  if (empty($data)) {
    $data =  FALSE;
  }

  return $data;
}

/**
 * hook emaudio_PROVIDER_embedded_link($code)
 * returns a link to view the video at the provider's site
 *  @param $code
 *    the string containing the audio code
 *  @return
 *    a string containing the URL to view the video at the original provider's site
 */
function emaudio_kiwi6_embedded_link($code) {
  return media_kiwi6_file_url($code);
}

/**
 * Implementation of emaudio_PROVIDER_audio().
 *
 * This actually displays the full/normal-sized player we want,
 * usually on the default page view.
 *
 * @param $code
 *   The code for the audio to embed.
 * @param $width
 *   The width to display the player.
 * @param $height
 *   The height to display the player.
 * @param $field
 *   The field info from the requesting node.
 * @param $item
 *   The actual content from the field.
 * @return
 *   The html of the embedded audio.
 */
function emaudio_kiwi6_audio($code, $width, $height, $field, $item) {
  return theme('emaudio_kiwi6_flash',
               $item['data'],
               $code,
               $field['widget']['audio_width'],
               $field['widget']['audio_autoplay']);
}

/**
 * Implementation of emaudio_PROVIDER_preview
 * this actually displays the preview-sized player we want,
 * commonly for the teaser
 * 
 *  @param $code
 *    The code for the audio to embed.
 *  @param $width
 *    The width to display the player.
 *  @param $height
 *    The height to display the player.
 *  @param $field
 *    The field info from the requesting node.
 *  @param $item
 *    The actual content from the field.
 *  @return
 *    The html of the embedded audio.
 */
function emaudio_kiwi6_preview($code, $width, $height, $field, $item) {
  return theme('emaudio_kiwi6_flash',
               $item['data'],
               $code,
               $field['widget']['preview_width'],
               $field['widget']['preview_autoplay']);
}

/**
 *  Implement emaudio_PROVIDER_content_generate().
 *
 *  Generates data for the devel module
 */
function emaudio_kiwi6_content_generate() {
  $items = array();

  $genres = array(
    'Alternative',
    'Blues',
    'Classical',
    'Country',
    'Electronic',
    'Hip+Hop',
    'Instrumental',
    'International',
    'Jazz',
    'Other',
    'Pop',
    'R%26B',
    'Rap',
    'Rock',
    'Techno',
    'Voice'
    );

  foreach ($genres as $genre) {
    $response = drupal_http_request('http://kiwi6.com/genres/' . $genre);

    if ($response->code == 200) {
      if (preg_match('@/file/(\S\w+)\?autostart@i', $response->data, $match)) {
        $items[] = media_kiwi6_file_url($match[1]);
      }
    }
  }

  return empty($items) ? FALSE : $items;
}

function theme_emaudio_kiwi6_flash($data, $code, $width, $autoplay) {
  $player = <<<HTML
<object type="application/x-shockwave-flash" data="http://kiwi6.com/swf/player.swf" id="audioplayer" height="24" width="%d" allowscriptaccess="always">
  <param name="movie" value="http://kiwi6.com/swf/player.swf" />
  <param name="FlashVars" value="playerID=audioplayer&soundFile=%s&autostart=%s&titles=%s" />
  <param name="quality" value="high" />
  <param name="menu" value="false" />
  <param name="allowscriptaccess" value="always" />
  <param name="wmode" value="transparent" />
</object>
HTML;

  if (variable_get('media_kiwi6_hostet_by_message', TRUE)
      OR variable_get('media_kiwi6_download_link', TRUE)) {
    $player .= '<div style="font:10px Arial,sans-serif;color:#aaa">';
    
    if (variable_get('media_kiwi6_hostet_by_message', TRUE)) {
      $player .= 'Hosted by <a style="color:#999" href="http://kiwi6.com">kiwi6.com file hosting</a>.';
    }

    if (variable_get('media_kiwi6_download_link', TRUE)) {
      $player .= sprintf('<a style="color:#999" href="%s">Download mp3</a> - <a href="http://kiwi6.com">Free File Hosting</a>.', media_kiwi6_file_url($code));
    }

    $player .= '</div>';
  }

  return sprintf($player, $width, $data['soundFile'], ($autoplay ? 'yes' : 'no'), $data['titles']);
}